#version 430 core
layout (location = 0) in vec3 position; 
layout (location = 1) in vec3 normal;

uniform vec3 color;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;

uniform mat3 normalMatrix;


out vec4 Color;
out vec3 FragPos;
out vec3 Normal;


uniform float time;
const float amplitude = 0.35;
const float frequency = 0.7;
const float PI = 3.14159;

vec3 newNormal = vec3(0, 1, 0);


void main()
{ 
   float distance = length(position);
  //float y = amplitude*sin(-PI*distance*frequency+time);

    //calculating y with Fractal Brownian Motion
    float y = sin(distance * frequency);
    float t = 0.005*(-time*130.0);
    y += sin(distance*frequency*2.1 + t)*4.5;
    y += sin(distance*frequency*1.72 + t*1.121)*4.0;
    y += sin(distance*frequency*2.221 + t*0.437)*5.0;
    y += sin(distance*frequency*3.1122+ t*4.269)*2.5;
    y *= amplitude*0.06;

    vec3 newposition =  vec3(position.x, y, position.z);     
    FragPos = vec3(modelMatrix * vec4(newposition,1));


    //calculate deviation for new normals
    float der_y = frequency * cos(distance * frequency);
    t = 0.005*(-time*130.0);
    der_y += frequency*2.1 * cos(distance*frequency*2.1 + t)*4.5;
    der_y += frequency*1.72 * cos(distance*frequency*1.72 + t*1.121)*4.0;
    der_y += frequency*2.221 * cos(distance*frequency*2.221 + t*0.437)*5.0;
    der_y += frequency*3.1122 * cos(distance*frequency*3.1122+ t*4.269)*2.5;
    der_y *= amplitude*0.06;

    if(der_y == 0){
            newNormal = vec3(0, 1, 0);
    }
    else{
        if(der_y >0){ //looking towards origin
           newNormal = vec3(-position.x, 1/der_y, -position.z);

        }
        else{ //looking away from origin
            newNormal = vec3(position.x, -1/der_y, position.z);
        }
    }

    Normal =  mat3(transpose(inverse(modelMatrix))) * normalize(newNormal);  


    Color =  vec4(color, 0.3);
   // gl_Position = viewProjMatrix* vec4(FragPos,1);
   gl_Position = viewProjMatrix * modelMatrix * vec4(position.x, y, position.z, 1.0);
}


