#version 430 core

layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 Brightness;

in vec3 FragPos;
in vec2 TexCoords;
in vec3 Normal;
in vec4 Color;

uniform vec3 highlightColor;

uniform sampler2D texture_diffuse1;
//uniform sampler2D texture_diffuse2;
//uniform sampler2D texture_diffuse3;
uniform sampler2D texture_specular1;
//uniform sampler2D texture_specular2;
//uniform sampler2D texture_specular3;

uniform float far_plane;
uniform samplerCube depthMap;
uniform bool shadows;

/*struct Material {
    sampler2D diffuse;
    sampler2D specular;    
    float shininess;
}; */
uniform float shininess;


struct Light {
    vec3 position;

    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};


uniform vec3 viewPos;
//uniform Material material;
uniform Light light;

// array of offset direction for sampling
vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

float ShadowCalculation(vec3 fragPos)
{
    // get vector between fragment position and light position
    vec3 fragToLight = fragPos - light.position;

    // define the closest depth - will be changed in the loop.    
    float closestDepth = texture(depthMap, fragToLight).r;
    
    // now get current linear depth as the length between the fragment and light position
    float currentDepth = length(fragToLight);

    float shadow = 0.0;
    float bias = 0.6;
    int samples = 20;
    float viewDistance = length(viewPos - fragPos);
    float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
    for(int i = 0; i < samples; ++i)
    {
        closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
        closestDepth *= far_plane;   // undo mapping [0;1]
        if(currentDepth - bias > closestDepth)
            shadow += 1.0;
    }
    shadow /= float(samples);
        
    // display closestDepth as debug (to visualize depth cubemap)
    FragColor = vec4(vec3(closestDepth / far_plane), 1.0);    
        
    return shadow;
}


void main()
{
    // ambient
    vec3 ambient = light.ambient * texture(texture_diffuse1, TexCoords).rgb;
  	
    // diffuse 
    vec3 norm = normalize(Normal);
    vec3 lightDir = normalize(light.position - FragPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = light.diffuse * diff * texture(texture_diffuse1, TexCoords).rgb;  
    
    // specular
    vec3 viewDir = normalize(viewPos - FragPos);
    vec3 reflectDir = reflect(-lightDir, norm);  
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), shininess);
    vec3 specular = light.specular * spec * texture(texture_specular1, TexCoords).rgb;  
    float shadow = shadows ? ShadowCalculation(FragPos) : 0.0;  
    vec3 result = ambient + (1.0 - shadow) * (diffuse + specular) + highlightColor;
   
    FragColor = vec4(result, 1.0) * Color;
    //FragColor = vec4(result, 1.0);




        float bright = dot(vec3(FragColor.x,FragColor.y,FragColor.z), vec3(0.7, 0.7, 0.7));
    if(bright > 1.0)
        Brightness = FragColor;
    else
        Brightness = vec4(0.0, 0.0, 0.0, 1.0);
} 